// src/extraScripts/svrpasswd.js
var readline = require("readline");
var fs = require("fs");
var crypto = {};
try {
  crypto = require("crypto");
} catch (ex) {
  crypto = {};
  crypto.__disabled__ = null;
  crypto.createHash = (type) => {
    if (type != "SHA256") throw new Error("Hash type not supported!");
    return {
      msg: "",
      update: (a) => {
        exports.msg = a;
        return exports;
      },
      digest: (ty) => {
        const chrsz = 8;
        const hexcase = 0;
        const safeAdd = (x, y) => {
          const lsw = (x & 65535) + (y & 65535);
          const msw = (x >> 16) + (y >> 16) + (lsw >> 16);
          return msw << 16 | lsw & 65535;
        };
        const S = (X, n) => {
          return X >>> n | X << 32 - n;
        };
        const R = (X, n) => {
          return X >>> n;
        };
        const Ch = (x, y, z) => {
          return x & y ^ ~x & z;
        };
        const Maj = (x, y, z) => {
          return x & y ^ x & z ^ y & z;
        };
        const Sigma0256 = (x) => {
          return S(x, 2) ^ S(x, 13) ^ S(x, 22);
        };
        const Sigma1256 = (x) => {
          return S(x, 6) ^ S(x, 11) ^ S(x, 25);
        };
        const Gamma0256 = (x) => {
          return S(x, 7) ^ S(x, 18) ^ R(x, 3);
        };
        const Gamma1256 = (x) => {
          return S(x, 17) ^ S(x, 19) ^ R(x, 10);
        };
        function coreSha256(m, l) {
          const K = new Array(
            1116352408,
            1899447441,
            3049323471,
            3921009573,
            961987163,
            1508970993,
            2453635748,
            2870763221,
            3624381080,
            310598401,
            607225278,
            1426881987,
            1925078388,
            2162078206,
            2614888103,
            3248222580,
            3835390401,
            4022224774,
            264347078,
            604807628,
            770255983,
            1249150122,
            1555081692,
            1996064986,
            2554220882,
            2821834349,
            2952996808,
            3210313671,
            3336571891,
            3584528711,
            113926993,
            338241895,
            666307205,
            773529912,
            1294757372,
            1396182291,
            1695183700,
            1986661051,
            2177026350,
            2456956037,
            2730485921,
            2820302411,
            3259730800,
            3345764771,
            3516065817,
            3600352804,
            4094571909,
            275423344,
            430227734,
            506948616,
            659060556,
            883997877,
            958139571,
            1322822218,
            1537002063,
            1747873779,
            1955562222,
            2024104815,
            2227730452,
            2361852424,
            2428436474,
            2756734187,
            3204031479,
            3329325298
          );
          let HASH = new Array(
            1779033703,
            3144134277,
            1013904242,
            2773480762,
            1359893119,
            2600822924,
            528734635,
            1541459225
          );
          let W = new Array(64);
          let a, b, c, d, e, f, g, h;
          let T1, T2;
          m[l >> 5] |= 128 << 24 - l % 32;
          m[(l + 64 >> 9 << 4) + 15] = l;
          for (let i2 = 0; i2 < m.length; i2 += 16) {
            a = HASH[0];
            b = HASH[1];
            c = HASH[2];
            d = HASH[3];
            e = HASH[4];
            f = HASH[5];
            g = HASH[6];
            h = HASH[7];
            for (let j = 0; j < 64; j++) {
              if (j < 16) W[j] = m[j + i2];
              else
                W[j] = safeAdd(
                  safeAdd(
                    safeAdd(Gamma1256(W[j - 2]), W[j - 7]),
                    Gamma0256(W[j - 15])
                  ),
                  W[j - 16]
                );
              T1 = safeAdd(
                safeAdd(safeAdd(safeAdd(h, Sigma1256(e)), Ch(e, f, g)), K[j]),
                W[j]
              );
              T2 = safeAdd(Sigma0256(a), Maj(a, b, c));
              h = g;
              g = f;
              f = e;
              e = safeAdd(d, T1);
              d = c;
              c = b;
              b = a;
              a = safeAdd(T1, T2);
            }
            HASH[0] = safeAdd(a, HASH[0]);
            HASH[1] = safeAdd(b, HASH[1]);
            HASH[2] = safeAdd(c, HASH[2]);
            HASH[3] = safeAdd(d, HASH[3]);
            HASH[4] = safeAdd(e, HASH[4]);
            HASH[5] = safeAdd(f, HASH[5]);
            HASH[6] = safeAdd(g, HASH[6]);
            HASH[7] = safeAdd(h, HASH[7]);
          }
          return HASH;
        }
        const str2binb = (str2) => {
          let bin = Array();
          const mask = (1 << chrsz) - 1;
          for (let i2 = 0; i2 < str2.length * chrsz; i2 += chrsz) {
            bin[i2 >> 5] |= (str2.charCodeAt(i2 / chrsz) & mask) << 24 - i2 % 32;
          }
          return bin;
        };
        const Utf8Encode = (string) => {
          string = string.replace(/\r\n/g, "\n");
          let utftext = "";
          for (let n = 0; n < string.length; n++) {
            let c = string.charCodeAt(n);
            if (c < 128) {
              utftext += String.fromCharCode(c);
            } else if (c > 127 && c < 2048) {
              utftext += String.fromCharCode(c >> 6 | 192);
              utftext += String.fromCharCode(c & 63 | 128);
            } else {
              utftext += String.fromCharCode(c >> 12 | 224);
              utftext += String.fromCharCode(c >> 6 & 63 | 128);
              utftext += String.fromCharCode(c & 63 | 128);
            }
          }
          return utftext;
        };
        const binb2hex = (binarray) => {
          const hexTab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
          let str2 = "";
          for (let i2 = 0; i2 < binarray.length * 4; i2++) {
            str2 += hexTab.charAt(
              binarray[i2 >> 2] >> (3 - i2 % 4) * 8 + 4 & 15
            ) + hexTab.charAt(binarray[i2 >> 2] >> (3 - i2 % 4) * 8 & 15);
          }
          return str2;
        };
        let s = Utf8Encode(exports.msg);
        let str = binb2hex(coreSha256(str2binb(s), s.length * chrsz));
        if (ty == "hex") return str;
        let hx = [];
        for (var i = 0; i < str.length; i += 2) {
          hx.push(parseInt(str[i] + str[i + 1], 16));
        }
        return Buffer.from(hx);
      }
    };
  };
}
if (!crypto.randomInt) {
  crypto.randomInt = (min, max) => {
    return Math.round(Math.random() * (max - min)) + min;
  };
}
var configJSON = {};
if (fs.existsSync(__dirname + "/config.json")) {
  let configJSONf = "";
  try {
    configJSONf = fs.readFileSync(__dirname + "/config.json");
  } catch (ex) {
    throw new Error("Cannot read JSON file.");
  }
  try {
    configJSON = JSON.parse(configJSONf);
  } catch (ex) {
    throw new Error("JSON Parse error.");
  }
}
var users = [];
if (configJSON.users != void 0) users = configJSON.users;
function saveConfig() {
  let configJSONobj = {};
  if (fs.existsSync(__dirname + "/config.json"))
    configJSONobj = JSON.parse(
      fs.readFileSync(__dirname + "/config.json").toString()
    );
  configJSONobj.users = users;
  const configString = JSON.stringify(configJSONobj, null, 2);
  fs.writeFileSync(__dirname + "/config.json", configString);
}
var args = process.argv;
var user = "";
var action = "change";
var forcechange = false;
if (process.argv.length <= (process.argv[0].indexOf("node") > -1 || process.argv[0].indexOf("bun") > -1 ? 2 : 1))
  args.push("-h");
for (let i = process.argv[0].indexOf("node") > -1 || process.argv[0].indexOf("bun") > -1 ? 2 : 1; i < args.length; i++) {
  if (args[i] == "-h" || args[i] == "--help" || args[i] == "-?" || args[i] == "/h" || args[i] == "/?") {
    console.log("SVR.JS user tool usage:");
    console.log(
      "node svrpasswd.js [-h] [--help] [-?] [/h] [/?] [-x] [-a|--add|-d|--delete] <username>"
    );
    console.log("-h -? /h /? --help    -- Displays help");
    console.log("-a --add              -- Add an user");
    console.log("-d --delete           -- Deletes an user");
    console.log("-x                    -- Changes hash algorithm");
    process.exit(0);
  } else if (args[i] == "-a" || args[i] == "--add") {
    if (action != "change") {
      console.log("Multiple actions specified.");
      console.log(
        "node svrpasswd.js [-h] [--help] [-?] [/h] [/?] [-x] [-a|--add|-d|--delete] <username>"
      );
      console.log("-h -? /h /? --help    -- Displays help");
      console.log("-a --add              -- Add an user");
      console.log("-d --delete           -- Deletes an user");
      console.log("-x                    -- Changes hash algorithm");
      process.exit(1);
    }
    action = "add";
  } else if (args[i] == "-d" || args[i] == "--delete") {
    if (action != "change") {
      console.log("Multiple actions specified.");
      console.log(
        "node svrpasswd.js [-h] [--help] [-?] [/h] [/?] [-x] [-a|--add|-d|--delete] <username>"
      );
      console.log("-h -? /h /? --help    -- Displays help");
      console.log("-a --add              -- Add an user");
      console.log("-d --delete           -- Deletes an user");
      console.log("-x                    -- Changes hash algorithm");
      process.exit(1);
    }
    action = "delete";
  } else if (args[i] == "-x") {
    if (forcechange) {
      console.log("Multiple -x options specified.");
      console.log(
        "node svrpasswd.js [-h] [--help] [-?] [/h] [/?] [-x] [-a|--add|-d|--delete] <username>"
      );
      console.log("-h -? /h /? --help    -- Displays help");
      console.log("-a --add              -- Add an user");
      console.log("-d --delete           -- Deletes an user");
      console.log("-x                    -- Changes hash algorithm");
      process.exit(1);
    }
    forcechange = true;
  } else {
    if (user != "") {
      console.log("Multiple users specified.");
      console.log(
        "node svrpasswd.js [-h] [--help] [-?] [/h] [/?] [-x] [-a|--add|-d|--delete] <username>"
      );
      console.log("-h -? /h /? --help    -- Displays help");
      console.log("-a --add              -- Add an user");
      console.log("-d --delete           -- Deletes an user");
      console.log("-x                    -- Changes hash algorithm");
      process.exit(1);
    }
    user = args[i];
  }
}
if (user == "") {
  console.log("No user specified.");
  console.log(
    "node svrpasswd.js [-h] [--help] [-?] [/h] [/?] [-x] [-a|--add|-d|--delete] <username>"
  );
  console.log("-h -? /h /? --help    -- Displays help");
  console.log("-a --add              -- Add an user");
  console.log("-d --delete           -- Deletes an user");
  console.log("-x                    -- Changes hash algorithm");
  process.exit(1);
}
function getUserIndex(username) {
  let ind = -1;
  for (let i = 0; i < users.length; i++) {
    if (users[i].name == username) {
      ind = i;
      break;
    }
  }
  return ind;
}
function sha256(msg) {
  let hash = crypto.createHash("SHA256");
  hash.update(msg);
  return hash.digest("hex");
}
function generateSalt() {
  let token = "";
  const strlist = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
  for (let i = 0; i < 63; i++) {
    token += strlist[crypto.randomInt(0, strlist.length)];
  }
  return token;
}
function password(callback) {
  const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
    prompt: "Password: "
  });
  rl.prompt();
  process.stdout.writeold = process.stdout.write;
  process.stdout.write = (s) => {
    process.stdout.writeold(s.replace(/[^\r\n]/g, ""));
  };
  rl.once("line", (line) => {
    process.stdout.write = process.stdout.writeold;
    var rl2 = readline.createInterface({
      input: process.stdin,
      output: process.stdout,
      prompt: "Confirm password: "
    });
    rl2.prompt();
    process.stdout.writeold = process.stdout.write;
    process.stdout.write = (s) => {
      process.stdout.writeold(s.replace(/[^\r\n]/g, ""));
    };
    rl2.on("line", (line2) => {
      process.stdout.write = process.stdout.writeold;
      rl2.close();
      if (line != line2) callback(false);
      else callback(line);
    });
  });
}
function promptAlgorithms(callback, bypass, pbkdf2, scrypt) {
  if (bypass) {
    if (scrypt) {
      callback("scrypt");
    } else if (pbkdf2) {
      callback("pbkdf2");
    } else {
      callback("sha256");
    }
    return;
  }
  let algorithms = {
    sha256: "Salted SHA256 (1 iteration) - fastest and uses least memory, but less secure",
    pbkdf2: "PBKDF2 (PBKDF2-HMAC-SHA512, 36250 iterations) - more secure and uses less memory, but slower",
    scrypt: "scrypt (N=2^14, r=8, p=1) - faster and more secure, but uses more memory"
  };
  if (!crypto.pbkdf2 || process.isBun && !(process.versions.bun && !process.versions.bun.match(
    /^(?:0\.|1\.0\.|1\.1\.[0-9](?![0-9])|1\.1\.1[0-2](?![0-9]))/
  )))
    delete algorithms.pbkdf2;
  const algorithmNames = Object.keys(algorithms);
  if (algorithmNames.length < 2) callback(algorithmNames[0]);
  console.log("Select password hashing algorithm. Available algorithms:");
  for (var i = 0; i < algorithmNames.length; i++) {
    console.log(algorithmNames[i] + " - " + algorithms[algorithmNames[i]]);
  }
  const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
    prompt: "Algorithm: "
  });
  rl.prompt();
  rl.on("line", (line) => {
    rl.close();
    line = line.trim();
    if (!algorithms[line]) callback(false);
    else callback(line);
  });
}
var userindex = getUserIndex(user);
if (action == "add" && userindex != -1) {
  console.log("User alerady exists.");
  process.exit(1);
} else if (action != "add" && userindex == -1) {
  console.log("User doesn't exist.");
  process.exit(1);
}
if (action == "delete") {
  users.splice(userindex, 1);
  saveConfig();
  console.log("User deleted successfully");
} else if (action == "add") {
  promptAlgorithms((algorithm) => {
    if (!algorithm) {
      console.log("Invalid algorithm!");
      process.exit(1);
    } else {
      password((password2) => {
        if (!password2) {
          console.log("Passwords don't match!");
          process.exit(1);
        } else {
          const salt = generateSalt();
          let hash = "";
          if (algorithm == "scrypt") {
            hash = crypto.scryptSync(password2, salt, 64).toString("hex");
          } else if (algorithm == "pbkdf2") {
            hash = crypto.pbkdf2Sync(password2, salt, 36250, 64, "sha512").toString("hex");
          } else {
            hash = sha256(password2 + salt);
          }
          users.push({
            name: user,
            pass: hash,
            salt,
            pbkdf2: algorithm == "pbkdf2" ? true : void 0,
            scrypt: algorithm == "scrypt" ? true : void 0,
            __svrpasswd_l2: true
          });
          saveConfig();
          console.log("User added successfully");
        }
      });
    }
  });
} else {
  promptAlgorithms(
    (algorithm) => {
      if (!algorithm) {
        console.log("Invalid algorithm!");
        process.exit(1);
      } else {
        password((password2) => {
          if (!password2) {
            console.log("Passwords don't match!");
            process.exit(1);
          } else {
            var salt = generateSalt();
            var hash = "";
            if (algorithm == "scrypt") {
              hash = crypto.scryptSync(password2, salt, 64).toString("hex");
            } else if (algorithm == "pbkdf2") {
              hash = crypto.pbkdf2Sync(password2, salt, 36250, 64, "sha512").toString("hex");
            } else {
              hash = sha256(password2 + salt);
            }
            users[userindex] = {
              name: user,
              pass: hash,
              salt,
              pbkdf2: algorithm == "pbkdf2" ? true : void 0,
              scrypt: algorithm == "scrypt" ? true : void 0,
              __svrpasswd_l2: true
            };
            saveConfig();
            console.log("Password changed successfully");
          }
        });
      }
    },
    users[userindex].__svrpasswd_l2 && !forcechange,
    users[userindex].pbkdf2,
    users[userindex].scrypt
  );
}
